from typing import List, Dict, Tuple
import flask
import mysql.connector
import json
import uuid
import bcrypt
import decimal
"""
Main file that creates the server and all routes as well as the database queries.
Note that the server is using self certified https. This certificate will become invalid sometime in 2022 so worth
checking if issues occur.
"""

app = flask.Flask(__name__)
config = {
        'user': 'root',
        'password': 'root',
        'host': 'db',
        'port': '3306',
        'database': 'drip_drones'
    }


# Execute select SQL query on DB and return result
def select_from_db(query, data = ()) -> List[Dict]:
    connection = mysql.connector.connect(**config)
    cursor = connection.cursor()
    cursor.execute(query, data)

    row_headers = [x[0] for x in cursor.description]  # this will extract row headers
    rv = cursor.fetchall()
    result_array = []
    for result in rv:
        result_array.append(dict(zip(row_headers, result)))

    cursor.close()
    connection.close()
    return result_array


# Execute arbitrary SQL query on DB
def execute_on_db(query, data):
    connection = mysql.connector.connect(**config)
    cursor = connection.cursor()
    cursor.execute(query, data)
    connection.commit()
    latest_row_id = cursor.lastrowid
    cursor.close()
    connection.close()
    return latest_row_id


class CustomJsonEncoder(json.JSONEncoder):

    def default(self, obj):
        if isinstance(obj, decimal.Decimal):
            return float(obj)
        return super(CustomJsonEncoder, self).default(obj)


# test page
@app.route('/', methods=['GET'])
def hello():
    return '<!DOCTYPE html>\n' +\
            '<html>\n' +\
            '<body>\n' +\
            '<p>Register Drone</p>\n' +\
            '<form name="dataForm">\n' +\
            '<label for="name">Enter Name:</label>\n' +\
            '<input type="text" id="name" name="name"><br>\n' +\
            '<label for="id">Enter ID:</label>\n' +\
            '<input type="text" id="id" name="id"><br>\n' +\
            '<label for="model">Enter Model:</label>\n' +\
            '<input type="text" id="model" name="model"><br>\n' +\
            '<label for="key"> Enter Public Key:</label>\n' +\
            '<input type="text" id="key" name="key"><br>\n' +\
            '</form>\n' +\
            '<button onclick="submitform()">Submit</button>\n' +\
            '<script>\n' +\
            '    function submitform() {\n' +\
            '        var name_val = document.getElementById("name").value;\n' +\
            '        var key_val = document.getElementById("key").value;\n' +\
            '        var model_val = document.getElementById("model").value;\n' +\
            '        var id_val = document.getElementById("id").value;\n' +\
            '        var jsonData = {\n' +\
            '            "owner_name": name_val, "pub_key": key_val, "model": model_val, "drip_id": id_val\n' +\
            '        }\n' +\
            '        let url = window.location.protocol + "//" + window.location.host + "/insert/drone";' +\
            '        var xhr = new XMLHttpRequest();\n' +\
            '        xhr.open("POST", url, true);\n' +\
            '        xhr.setRequestHeader("Content-Type", "application/json; charset=UTF-8");\n' +\
            '        xhr.send(JSON.stringify(jsonData));\n' +\
            '    }\n' +\
            '</script>\n' +\
            '</body>\n' +\
            '</html>'



# Returns all the drones in the table
@app.route('/get/drones/all', methods=['GET'])
def get_all_drones() -> Tuple[List[dict], int, Dict[str, str]]:
    return json.dumps(select_from_db('SELECT * FROM drone'), indent=4, sort_keys=True, default=str, cls=CustomJsonEncoder), 200, {'Content-Type': 'application/json; charset=utf-8'}


# Returns those drones whose drone_id match with the ones in json array
@app.route('/get/drones', methods=['POST'])
def get_drones() -> Tuple[List[dict], int, Dict[str, str]]:
    json_req = flask.request.json
    res = select_from_db('SELECT * FROM drone')
    filtered_array = []

    for row in res:
        if row['drip_id'] in json_req['drip_ids']:
            filtered_array.append(row)

    return json.dumps(filtered_array, indent=4, sort_keys=True, default=str, cls=CustomJsonEncoder), 200, {'Content-Type': 'application/json; charset=utf-8'}


# Returns the public key for drone whose drone_id match with the one in json array
@app.route('/get/key', methods=['POST'])
def get_drone_keys() -> Tuple[List[dict], int, Dict[str, str]]:
    json_req = flask.request.json
    res = select_from_db('SELECT * FROM drone')
    filtered_array = []

    for row in res:
        if row['drip_id'] in json_req['drip_ids']:
            filtered_array.append({"drip_id": row['drip_id'], "pub_key": row['pub_key']})

    return json.dumps(filtered_array, indent=4, sort_keys=True, default=str, cls=CustomJsonEncoder), 200, {'Content-Type': 'application/json; charset=utf-8'}


# Inserts a drone in table
@app.route('/insert/drone', methods=['POST'])
def insert_drone() -> Tuple[List[dict], int, Dict[str, str]]:
    json_req = flask.request.json
    print(json_req)
    if "lat" in json_req and "lon" in json_req:
        query = "REPLACE INTO drone(drip_id, model, owner_name, pub_key, lat, lon) VALUES (%s, %s, %s, %s, %s, %s)"
        data = (json_req['drip_id'], json_req['model'], json_req['owner_name'], json_req['pub_key'], json_req['lat'],
                json_req['lon'])
        execute_on_db(query, data)
    else:
        query = "REPLACE INTO drone(drip_id, model, owner_name, pub_key) VALUES (%s, %s, %s, %s)"
        data = (json_req['drip_id'], json_req['model'], json_req['owner_name'], json_req['pub_key'])
        execute_on_db(query, data)
    return json.dumps(select_from_db('SELECT * FROM drone'), indent=4, sort_keys=True, default=str, cls=CustomJsonEncoder), 200, {'Content-Type': 'application/json; charset=utf-8'}


if __name__ == '__main__':
    app.run(host='0.0.0.0', port=5000, ssl_context=('cert.pem', 'key.pem'))
